#include "Server.h"
#include "utility/w5500.h"
#include "utility/socket.h"

void EthernetMaketServer::setServerHandler(std::function<void(EthernetMaketClient)> handler)
{
    serverHandler = handler;
}

void EthernetMaketServer::resetServerHandler()
{
    serverHandler = [](EthernetMaketClient sclient) {};
}

EthernetMaketClient EthernetMaketServer::available(int sock)
{
    accept(sock);
    EthernetMaketClient client(sock);
    if (EthernetClass::_server_port[sock] == _port &&
        (client.status() == SnSR::ESTABLISHED ||
         client.status() == SnSR::CLOSE_WAIT))
    {
        if (client.available())
        {
            return client;
        }
    }
    return EthernetMaketClient(MAX_SOCK_NUM);
}

void EthernetMaketServer::accept(int sock)
{
    int listening = 0;
    EthernetClient client(sock);

    if (EthernetClass::_server_port[sock] == _port)
    {
        if (client.status() == SnSR::LISTEN)
        {
            listening = 1;
        }
        else if (client.status() == SnSR::CLOSE_WAIT && !client.available())
        {
            client.stop();
        }
    }

    if (!listening)
    {
        // begin();
        begin(sock); // added
    }
}

void EthernetMaketServer::begin(int sock)
{
    EthernetClient client(sock);
    if (client.status() == SnSR::CLOSED)
    {
        socket(sock, SnMR::TCP, _port, 0);
        listen(sock);
        EthernetClass::_server_port[sock] = _port;
    }
}